unless window.coffeetable then window.coffeetable = {}
app = window.coffeetable

app.beatmap = {

	canvas: -> 
		document.getElementById('beatmap')

	parse_func_name: (ctx, func_name, coords) ->

		###

		Takes a canvas context, a string referring to the name of a function to execute
		within that context, and a list of coordinates to pass to the designated function.
		The function is executed within the context.

		###
		if func_name in ["moveTo", "lineTo"]
			[x, y] = coords
			if func_name == "moveTo"
				ctx.moveTo(x, y)
			else
				ctx.lineTo(x, y)
		else if func_name == "quadraticCurveTo"
			[ctx_x, ctx_y, x, y] = coords
			ctx.quadraticCurveTo(ctx_x, ctx_y, x, y)
		else
			switch func_name 
				when "fill" then ctx.fill()
				when "beginPath" then ctx.beginPath()
				when "closePath" then ctx.closePath()
				when "stroke" then ctx.stroke()
	
	draw_envelope: (beat_info, color) ->

		###

		Takes an array of drawing instructions (see `.../views/beat_info.clj`), and
		executes them on the canvas. Color may optionally be specified (defaults to
		yellow).

		###
		unless color then color = "black"
		ctx = app.beatmap.canvas().getContext('2d')
		ctx.strokeStyle = ctx.fillStyle = color
		ctx.lineWidth = 1
		ctx.lineJoin = 'round'
		_.each(beat_info,
			(instrs) =>
				_.each(instrs, 
					(instr) =>
						app.beatmap.parse_func_name(ctx, instr[0], _.rest(instr))))

	refresh_canvas: ->

		###

		Covers the canvas with a blue rectangle: a clean slate.

		###

		ctx = app.beatmap.canvas().getContext('2d')
		ctx.fillStyle = "white"
		ctx.fillRect(0, 0, app.beatmap.canvas.width, app.beatmap.canvas.height)

	reset: () ->

		###

		Resets the height of the canvas to allow for a new measure count, and triggers
		`refresh_canvas`.

		###

		new_height = 120 * app.parameters.measures()
		unless new_height == app.beatmap.canvas.height 
			$('#beatmap').attr('height', new_height)
		app.beatmap.refresh_canvas()

	redraw: =>

		###

		Gets the current measure count, the url to get drawing instructions from, and
		finally the drawing instructions from the server. The instructions are
		passed to `draw_envelope` after the canvas is reset and the response recieved.

		###

		app.beatmap.reset()
		m = app.parameters.measures()
		res = app.parameters.resolution()
		[a, d, s, r] = app.parameters.envelope()
		active_beats = app.beats.get_active_beats()
		instrs = app.beats.get_instrs(m, res, a, d, s, r, active_beats)
		if instrs then app.beatmap.draw_envelope(instrs)
}