(function() {
  var app,
    _this = this;

  if (!window.coffeetable) window.coffeetable = {};

  app = window.coffeetable;

  app.beatmap = {
    canvas: function() {
      return document.getElementById('beatmap');
    },
    parse_func_name: function(ctx, func_name, coords) {
      /*
      
      		Takes a canvas context, a string referring to the name of a function to execute
      		within that context, and a list of coordinates to pass to the designated function.
      		The function is executed within the context.
      */
      var ctx_x, ctx_y, x, y;
      if (func_name === "moveTo" || func_name === "lineTo") {
        x = coords[0], y = coords[1];
        if (func_name === "moveTo") {
          return ctx.moveTo(x, y);
        } else {
          return ctx.lineTo(x, y);
        }
      } else if (func_name === "quadraticCurveTo") {
        ctx_x = coords[0], ctx_y = coords[1], x = coords[2], y = coords[3];
        return ctx.quadraticCurveTo(ctx_x, ctx_y, x, y);
      } else {
        switch (func_name) {
          case "fill":
            return ctx.fill();
          case "beginPath":
            return ctx.beginPath();
          case "closePath":
            return ctx.closePath();
          case "stroke":
            return ctx.stroke();
        }
      }
    },
    draw_envelope: function(beat_info, color) {
      /*
      
      		Takes an array of drawing instructions (see `.../views/beat_info.clj`), and
      		executes them on the canvas. Color may optionally be specified (defaults to
      		yellow).
      */
      var ctx,
        _this = this;
      if (!color) color = "black";
      ctx = app.beatmap.canvas().getContext('2d');
      ctx.strokeStyle = ctx.fillStyle = color;
      ctx.lineWidth = 1;
      ctx.lineJoin = 'round';
      return _.each(beat_info, function(instrs) {
        return _.each(instrs, function(instr) {
          return app.beatmap.parse_func_name(ctx, instr[0], _.rest(instr));
        });
      });
    },
    refresh_canvas: function() {
      /*
      
      		Covers the canvas with a blue rectangle: a clean slate.
      */
      var ctx;
      ctx = app.beatmap.canvas().getContext('2d');
      ctx.fillStyle = "white";
      return ctx.fillRect(0, 0, app.beatmap.canvas.width, app.beatmap.canvas.height);
    },
    reset: function() {
      /*
      
      		Resets the height of the canvas to allow for a new measure count, and triggers
      		`refresh_canvas`.
      */
      var new_height;
      new_height = 120 * app.parameters.measures();
      if (new_height !== app.beatmap.canvas.height) {
        $('#beatmap').attr('height', new_height);
      }
      return app.beatmap.refresh_canvas();
    },
    redraw: function() {
      /*
      
      		Gets the current measure count, the url to get drawing instructions from, and
      		finally the drawing instructions from the server. The instructions are
      		passed to `draw_envelope` after the canvas is reset and the response recieved.
      */
      var a, active_beats, d, instrs, m, r, res, s, _ref;
      app.beatmap.reset();
      m = app.parameters.measures();
      res = app.parameters.resolution();
      _ref = app.parameters.envelope(), a = _ref[0], d = _ref[1], s = _ref[2], r = _ref[3];
      active_beats = app.beats.get_active_beats();
      instrs = app.beats.get_instrs(m, res, a, d, s, r, active_beats);
      if (instrs) return app.beatmap.draw_envelope(instrs);
    }
  };

}).call(this);
