unless window.coffeetable then window.coffeetable = {}
app = window.coffeetable

app.button = {

	intervalIDs: []

	HTML: (data_label, label) -> 
		"""
		<h4>#{label}</h4>
		<canvas class="up"></canvas>
		<input type="text" list="#{data_label}" data-index="0">
		<canvas class="down"></canvas>
		"""

	POSITIONS: [{top: '5px'}, {bottom: '-10px'}]
	TYPES: ['up', 'down']

	make_datalist: (data, label) ->
		props = (['<option value="', opt, '"/>'] .join('') for opt in data)
		.join('')
		['<datalist id="', label, '">', props, '</datalist>']
		.join('')

	set_datalist: (container, data, label) ->
		data = app.button.make_datalist(data, label)
		$(container).append(data)

	reset_datalist: (data, label) ->
		id = '#' + label
		parent = $(id).parent()
		$(id).remove()
		app.button.set_datalist(parent, data, label)

	datalist_length: (label) ->
		$('option', label).length

	get_next_option: (target) ->
		datalist = '#' + $(target).attr('list')
		index = parseInt($(target).attr('data-index'))
		len = parseInt(app.button.datalist_length(datalist))
		new_i = if index + 1 >= len then 0 else index + 1
		$(target).val($($('option', datalist)[new_i]).attr('value'))
		$(target).attr('data-index', new_i)

	get_prev_option: (target) ->
		datalist = '#' + $(target).attr('list')
		index = parseInt($(target).attr('data-index'))
		len = parseInt(app.button.datalist_length(datalist))
		new_i = if index - 1 < 0 then len - 1 else index - 1
		$(target).val($($('option', datalist)[new_i]).attr('value'))
		$(target).attr('data-index', new_i)

	get_current_option: (target) ->
		datalist = '#' + $(target).attr('list')
		index = parseInt($(target).attr('data-index'))
		len = parseInt(app.button.datalist_length(datalist))
		i = if index >= len then len - 1 else index
		$(target).val($($('option', datalist)[i]).attr('value'))

	draw_triangle: (el, color, direction) ->

		###

		Draws a triangle on canvas `el`. 

		###

		if el.getContext
			h = el.height
			w = el.width
			[point, base] = if direction == 'up' then [0, h] else [h, 0]
			context = el.getContext('2d')
			context.beginPath()
			context.moveTo((w / 2), point)
			context.lineTo(0, base)
			context.lineTo(w, base)
			context.lineTo((w / 2), point) 
			context.fillStyle = if color then color else 'black'
			context.fill()

	button_click: (target, func, on_click) ->

		###

		Triggers `change_param`, and then causes `change_param` to repeat every 200ms.

		###

		func(target)
		app.button.intervalIDs.push(setInterval(func, 200, target))
		if on_click 
			on_click()
			app.button.intervalIDs.push(setInterval(on_click, 200))


	init: (container, data_label, label, color, on_click) ->

		###

		Causes up and down buttons to trigger `button_click` on mousedown, and clears
		the `change_param` interval on mouseup. Also initializes tooltips for up and down
		buttons.

		###

		color = color or 'black'
		$(container).html(app.button.HTML(data_label, label))
		target = $('input', container)
		app.button.get_current_option(target)

		handlers = [app.button.get_next_option, app.button.get_prev_option]

		on_clk_is_arr = if (typeof on_click) == 'object' then true else false

		$('.up, .down', container).each(
			(i, button) ->
				app.button.draw_triangle(this, color, app.button.TYPES[i])
				$(this).mousedown(->
					params = [
						target
						handlers[i]
						if on_clk_is_arr then on_click[i] else on_click] 
					app.button.button_click.apply(null, params))
				$(this).mouseup(-> 
					_.each(app.button.intervalIDs,
						(id, i) ->
 							clearInterval(id)
 							delete app.button.intervalIDs[i]))
		)
}