(function() {
  var app;

  if (!window.coffeetable) window.coffeetable = {};

  app = window.coffeetable;

  app.button = {
    intervalIDs: [],
    HTML: function(data_label, label) {
      return "<h4>" + label + "</h4>\n<canvas class=\"up\"></canvas>\n<input type=\"text\" list=\"" + data_label + "\" data-index=\"0\">\n<canvas class=\"down\"></canvas>";
    },
    POSITIONS: [
      {
        top: '5px'
      }, {
        bottom: '-10px'
      }
    ],
    TYPES: ['up', 'down'],
    make_datalist: function(data, label) {
      var opt, props;
      props = ((function() {
        var _i, _len, _results;
        _results = [];
        for (_i = 0, _len = data.length; _i < _len; _i++) {
          opt = data[_i];
          _results.push(['<option value="', opt, '"/>'].join(''));
        }
        return _results;
      })()).join('');
      return ['<datalist id="', label, '">', props, '</datalist>'].join('');
    },
    set_datalist: function(container, data, label) {
      data = app.button.make_datalist(data, label);
      return $(container).append(data);
    },
    reset_datalist: function(data, label) {
      var id, parent;
      id = '#' + label;
      parent = $(id).parent();
      $(id).remove();
      return app.button.set_datalist(parent, data, label);
    },
    datalist_length: function(label) {
      return $('option', label).length;
    },
    get_next_option: function(target) {
      var datalist, index, len, new_i;
      datalist = '#' + $(target).attr('list');
      index = parseInt($(target).attr('data-index'));
      len = parseInt(app.button.datalist_length(datalist));
      new_i = index + 1 >= len ? 0 : index + 1;
      $(target).val($($('option', datalist)[new_i]).attr('value'));
      return $(target).attr('data-index', new_i);
    },
    get_prev_option: function(target) {
      var datalist, index, len, new_i;
      datalist = '#' + $(target).attr('list');
      index = parseInt($(target).attr('data-index'));
      len = parseInt(app.button.datalist_length(datalist));
      new_i = index - 1 < 0 ? len - 1 : index - 1;
      $(target).val($($('option', datalist)[new_i]).attr('value'));
      return $(target).attr('data-index', new_i);
    },
    get_current_option: function(target) {
      var datalist, i, index, len;
      datalist = '#' + $(target).attr('list');
      index = parseInt($(target).attr('data-index'));
      len = parseInt(app.button.datalist_length(datalist));
      i = index >= len ? len - 1 : index;
      return $(target).val($($('option', datalist)[i]).attr('value'));
    },
    draw_triangle: function(el, color, direction) {
      /*
      
      		Draws a triangle on canvas `el`.
      */
      var base, context, h, point, w, _ref;
      if (el.getContext) {
        h = el.height;
        w = el.width;
        _ref = direction === 'up' ? [0, h] : [h, 0], point = _ref[0], base = _ref[1];
        context = el.getContext('2d');
        context.beginPath();
        context.moveTo(w / 2, point);
        context.lineTo(0, base);
        context.lineTo(w, base);
        context.lineTo(w / 2, point);
        context.fillStyle = color ? color : 'black';
        return context.fill();
      }
    },
    button_click: function(target, func, on_click) {
      /*
      
      		Triggers `change_param`, and then causes `change_param` to repeat every 200ms.
      */      func(target);
      app.button.intervalIDs.push(setInterval(func, 200, target));
      if (on_click) {
        on_click();
        return app.button.intervalIDs.push(setInterval(on_click, 200));
      }
    },
    init: function(container, data_label, label, color, on_click) {
      /*
      
      		Causes up and down buttons to trigger `button_click` on mousedown, and clears
      		the `change_param` interval on mouseup. Also initializes tooltips for up and down
      		buttons.
      */
      var handlers, on_clk_is_arr, target;
      color = color || 'black';
      $(container).html(app.button.HTML(data_label, label));
      target = $('input', container);
      app.button.get_current_option(target);
      handlers = [app.button.get_next_option, app.button.get_prev_option];
      on_clk_is_arr = (typeof on_click) === 'object' ? true : false;
      return $('.up, .down', container).each(function(i, button) {
        app.button.draw_triangle(this, color, app.button.TYPES[i]);
        $(this).mousedown(function() {
          var params;
          params = [target, handlers[i], on_clk_is_arr ? on_click[i] : on_click];
          return app.button.button_click.apply(null, params);
        });
        return $(this).mouseup(function() {
          return _.each(app.button.intervalIDs, function(id, i) {
            clearInterval(id);
            return delete app.button.intervalIDs[i];
          });
        });
      });
    }
  };

}).call(this);
