(function() {
  var app,
    __slice = Array.prototype.slice;

  if (!window.coffeetable) window.coffeetable = {};

  app = window.coffeetable;

  app.pitch = {
    /*
    
    	Directly adapted from 
    	https://github.com/overtone/overtone/blob/master/src/overtone/music/pitch.clj
    */
    notes: app.util.cycle(['A', 'A#', 'B', 'C', 'C#', 'D', 'D#', 'E', 'F', 'F#', 'G', 'G#']),
    ionian: [2, 2, 1, 2, 2, 2, 1],
    major: 0,
    minor: 5,
    dorian: 1,
    phrygian: 2,
    lydian: 3,
    mixolydian: 4,
    aeolian: 5,
    locrian: 6,
    pentatonic: [2, 3, 2, 2, 3],
    minor_pentatonic: [2, 2, 3, 2, 3],
    major_pentatonic: [3, 2, 2, 3, 2],
    whole_tone: [2, 2, 2, 2, 2, 2],
    chromatic: [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
    harmonic_minor: [2, 1, 2, 2, 1, 3, 1],
    melodic_minor_asc: [2, 1, 2, 2, 2, 2, 1],
    hungarian_minor: [2, 1, 3, 1, 1, 3, 1],
    rotate_ionian: function(offset) {
      var ionian;
      ionian = app.util.cycle(app.pitch.ionian);
      return ionian.drop(offset).take(app.pitch.ionian.length);
    },
    get_root: function(note_name) {
      return app.pitch.notes.skip_to(note_name).take(app.pitch.notes.len);
    },
    get_scale: function(root, scale_name) {
      var i, j, notes, offset, pitches, steps;
      notes = app.pitch.get_root(root);
      pitches = [];
      if (_.contains(['pentatonic', 'minor_pentatonic', 'major_pentatonic', 'whole_tone', 'chromatic', 'harmonic_minor', 'melodic_minor_asc', 'hungarian_minor'], scale_name)) {
        steps = app.pitch[scale_name];
      } else {
        if (scale_name === 'ionian') {
          offset = 0;
        } else {
          offset = app.pitch[scale_name];
        }
        steps = app.pitch.rotate_ionian(offset);
      }
      i = 0;
      j = 0;
      while (i < steps.length) {
        pitches.push(notes[j]);
        j += steps[i];
        i += 1;
      }
      return pitches;
    },
    get_active_notes: function(root, scale_name, octave_range, steps_to_take, direction) {
      var active_notes, pitches, rest;
      pitches = app.pitch.get_scale(root, scale_name);
      if (direction === 'down') {
        root = pitches[0], rest = 2 <= pitches.length ? __slice.call(pitches, 1) : [];
        pitches = _.flatten([root, rest.reverse()]);
      }
      active_notes = [];
      _.each(octave_range, function(octave) {
        var i, _results;
        i = 0;
        _results = [];
        while (i < steps_to_take) {
          active_notes.push("" + pitches[i] + octave);
          _results.push(i += 1);
        }
        return _results;
      });
      return active_notes;
    }
  };

}).call(this);
