unless window.coffeetable then window.coffeetable = {}
app = window.coffeetable

app.util = {

	knobify: (el, min, max, val, on_change) ->

		###

		Takes an element (to be converted into a knob), minimum, maximum, and default
		values, and a function to call on change. The on_change function is delayed 250ms
		(see DelayedWithInterrupt, above).

		###

		deferer = app.util.delay_with_interrupt(on_change, 250)
		$(el).attr('value', val)
		$(el).knob({
			'width': 90
			'min': min
			'max': max
			'change': (v) -> deferer.start_timer(v)
		})

	delay_with_interrupt: (func, ms_delay) ->

		###

		Takes a function and the number of milliseconds to wait before executing it.
		Returns an object with `start_timer` and `cancel` methods; if `start_timer`
		is called again before `ms_delay` has transipired, the timer is reset.

		This is useful for the BPM knob, which needs to have a little slack in it's
		on_change function so that it isn't called too many times.

		###

		deferer = {
			start_timer: (params) ->	
				deferer.cancel()
				deferer.timeoutID = setTimeout(func, ms_delay, params)

			cancel: ->
				if typeof(deferer.timeoutID) == 'number'
					clearTimeout(deferer.timeoutID)
					delete deferer.timeoutID
		}

	cycle: (arr) ->
		cycle = {
			i: 0
			len: arr.length

			drop: (n) ->

				###

				Moves the index pointer `cycle.i` forward by `n`, looping back around to the beginning 
				if `n` is greater than the length of the array being cycled. 

				###	
				
				i = n + cycle.i
				if i < cycle.len
					if i > -1
						cycle.i = i 
					else
						cycle.i = cycle.len - 1
				else 
					cycle.i = i % cycle.len
				return this

			skip_to: (target) ->

				###

				Moves `cycle.i` to the index of `target` if `target` is in `arr`; returns `this`.

				###

				new_i = _.indexOf(arr, target)
				if new_i > -1
					cycle.i = new_i
				else
					console.log("Error: target is not in this array.")
				return this

			take: (n, skip) ->

				###

				Takes `n` elements from the cycled array, starting from wherever `cycle.i` has been set
				and looping back to the start of the array if necessary. 
				Either drops `skip` (if present) or resets `cycle.i` to `0` after completion.

				###

				j = 0
				results = []
				while j < n
					index = cycle.i + j
					if index >= cycle.len then index = index % cycle.len
					j += 1
					results.push(arr[index])
				if skip then cycle.drop(skip) else cycle.i = 0
				return results

			next: ->
				cycle.i += 1
				cycle.i = if cycle.i == cycle.len then 0 else cycle.i
				return arr[cycle.i]

			prev: ->
				cycle.i -= 1
				cycle.i = if cycle.i < 0 then cycle.len - 1 else cycle.i
				return arr[cycle.i]

			get_arr: -> arr
		}

	compare_nested_array: (active_beats, m, b) ->

		###

		Because, according to Javascript, `[[1, 1]][0] !== [1, 1]`.

		###

		in_array = false
		_.each(active_beats, (beat) ->
			[am, ab] = beat
			if am == m and ab == b then in_array = true)
		in_array
}
