# CoffeeTable

A rhythm machine in CoffeeScript. 

You can play with a demo at http://tborg.github.com/CoffeeTable

Please note: CoffeeTable doesn't actually generate any sound or MIDI right now. The UI components and visualizer are starting to come together. Changes you make to the paramaters at the top will be reflected in the visualizer at the bottom. I think it's fun to see what kind of patterns you can make with different combinations of rhythm generators.

## Introduction

CoffeeTable is intended to be an off-center take on an arpeggiator. Instead of clicking on a piano roll to specify pitch and rhythm patterns, like you would in many digital audio workstation programs today, with CoffeeTable you dial in arithmetic progressions which are combined to form rhythmic patterns.

I wanted to try it this way because I thought it would be nice to establish interesting grooves with one touch, rather than by having to pick them out manually on a piano roll. For the live electronic music performer, it's a clutch convenience.

This project was for a Summer 2012 independent study towards a M.A. in Digital Humanities at Loyola University Chicago, under Dr. George Thiruvathukal.

## Scripting Overview

This project consists of a single static web page. The functionality is wrapped up in a series of JavaScript modules (that I wrote in CoffeeScript). They are 'modules' in that each of the files listed below is responsible for its own little chunk of functionality, and consists of an object that is attached to window.coffeetable (i.e. is hooked into the global scope.)

I also used a couple of popular libraries: jQuery and Underscore. jQuery, among other things, makes it much easier to work with the DOM than the native methods for the same; Underscore adds convenient, 'functional' functions like `_.reduce` and `_.filter`. It also goes a long way towards ensuring browser compatibility.

The CSS for this project is pretty basic and could probably be a lot less hacky.

### [app](https://github.com/tborg/CoffeeTable/blob/master/js/app.coffee)

Actually, `app.coffee` isn't a module. This file executes on document ready, triggering the necessary initialization functions.

### [util](https://github.com/tborg/CoffeeTable/blob/master/js/util.coffee)

Contains a few nuts-and-bolts functions I used widely. The first two, `knobify` and `delayed-with-interrupt`, are an artifact from the early stage of the project, when I was using jquery knobs instead of the custom buttons I later designed. `delayed-with-interrupt` was necessary for ensuring that the knobs would update on change, but not until the value of the knob actually stopped changing for a specified number of milliseconds. These two functions are still here because I'm probably going to re-integrate some knobs at some point.

The most important and widely used function in this file is `cycle`, which wraps a given array in a closure, and provides some methods for traversing around the array in a loop indefinitely.

`util` also provides a function `compare_nested_array`, which was necessary because, according to Javascript, `[[1, 1]][0] !== [1, 1]`.
The function doesn't universally compare nested arrays, because I didn't need it to: it's tailored to expect the use case, in which the nested arrays are of length 2.

### [beatmap](https://github.com/tborg/CoffeeTable/blob/master/js/beatmap.coffee)

For the visualizer, which I call the 'beatmap', active beats are drawn on an HTML canvas as an envelope waveform and rests are drawn as flat lines. `Beatmap` provides methods for ensuring that the visualizer is up-to-date with the user-specified parameters.

The interesting thing about `beatmap` is that it actually expects drawing instructions as data: `draw_envelope` -- the function that draws on the canvas -- takes an array of data in which (like in a lisp) the function name is given first and any subsequent paramaters are applied to the function.

I developed it this way because originally I was calculating the drawing instructions on the server side and sending them to the client as JSON. But I found that it works nicely to separate the functions that produce drawing instructions and the functions that actually draw.

For a good example of what those drawing instructions look like, have a look at the functions `amp_env` and `rest_beat` in the `beats` module (below).

### [beats](https://github.com/tborg/CoffeeTable/blob/master/js/beats.coffee)

The functions in `beats` are responsible for reading the active rhythm generators, producing arithmetic progressions from them, combining the progressions with set operations (either union or intersection), and then using the single resultant progression to generate drawing instructions, which are executed by `beatmap`.

It was actually quite difficult for me to figure out how to recurse through the tree of progressions appropriately. In an earlier stage of this project, I had decided not to have a set number of rhythm generators, but instead to allow the user to create as many as desired to an arbitrary nesting depth. To simplify things (both from a UI perspective and from a coding perspective) I later settled on presenting 3 rows of rhythm generators each containing 3 generators which can be turned on or off. A given row can be combined with either union or intersection, and the rows are always then combined with union. 

### [button](https://github.com/tborg/CoffeeTable/blob/master/js/button.coffee)

As I said above, I started out using knobs, but I didn't like the knobs. So I made my own little generalized UI widget: `button`. This provides up and down arrows drawn on canvas elements, an input box, and a label. It cycles through data provided in a specified HTML5 datalist element. It also accepts `click` event handlers.

### [gen_handlers](https://github.com/tborg/CoffeeTable/blob/master/js/gen_handlers.coffee)

Provides some methods for initializing and activating the rhythm generators.

### [list_handlers](https://github.com/tborg/CoffeeTable/blob/master/js/list_handlers.coffee)

A few of the major paramaters (measures, resolution, and and/or type-toggles) are presented as lists with a specific functionality: only the selected value is shown. When the mouse enters the element, the list drops down. New selected values are removed from the list, the list collapses, and then the selected value is prepended to the list (and shown). This file provides the methods that make that happen.

### [parameters](https://github.com/tborg/CoffeeTable/blob/master/js/parameters.coffee)

This module provides access to and initialization of some global parameters: measures, resolution, and the envelope (attack, decay, sustain, and release sliders). It also provides a method (`reset_gen_max`) that ensures that the rhythm generators have a maximum value that will actually still produce some active beats (i.e. not greater than resolution * measures).

### [pitch](https://github.com/tborg/CoffeeTable/blob/master/js/pitch.coffee)

This module contains some music theory adapted more or less directly from the Overtone project (I ported it from Clojure).

The `cycle` function (from the `util` module) is used widely here to generate pitch class names given a root note and a scale name.

The visualizer will eventually also draw note names next to active beats; right now all it does is list them.

!!TODO: The octave numbers are coming out wrong. They need to go up or down (depending on the direction) past C rather than past the root note.

### [pitch_handlers](https://github.com/tborg/CoffeeTable/blob/master/js/pitch_handlers.coffee)

Functions for initializing and manipulating the pitch UI widgets.

### [tooltips](https://github.com/tborg/CoffeeTable/blob/master/js/tooltips.coffee)

Certain elements, on hover, cause tooltips to appear. This file manages that in a central location. The tooltips are underdeveloped.


